@@ -114,7 +114,13 @@ def _forward2afterward(forward_seq):
 
 
 # transform an string into OTS BoolQuery to set searching condition in metadata
-def _seq2boolquery(metadata_condition,include_children = True):
+def _seq2boolquery(metadata_condition,include_children = True, exlude_Unclassified = False):
+    if metadata_condition == "":
+        metadata_condition = "organ<>a" # all cells
+    
+    if(exlude_Unclassified):
+        metadata_condition = "(" + metadata_condition + ") && (cell_type <> Unclassified)"
+        
     ## [0601] transform uHAF_name to hcad_name
     metadata_condition = metadata_condition.replace("uHAF_name","hcad_name")
     # standardize string
@@ -519,7 +525,7 @@ def build_index():
         client_global.create_search_index(_tablename, "metadata", index_meta)
 
 
-def query_cells(metadata_conditions,include_children=False):
+def query_cells(metadata_conditions="",include_children=False,exlude_Unclassified=False):
     """query cells and return the list of their primary keys
     
         This function is used to query cells satisfing the metadata conditions in the OTS table on the server. The conditions on the metadata should be a structured string which is a combination of several logical expressions. 
@@ -552,6 +558,8 @@ def query_cells(metadata_conditions,include_children=False):
             'organ == Lung && !seq_tech == 10X  '
 
             'organ == Heart &&(cell_type == Fibrocyte || region <> atria)'
+        
+        The default value of this parameter is '', which will get all rows in the database.
 
     include_children: bool 
         include_children is the parameter deciding if subtypes should be included in the query with condition on cell type. For example, when we query "cell_type == T cell" and set include_children as True, 
@@ -568,7 +576,7 @@ def query_cells(metadata_conditions,include_children=False):
 
         The primary key list can be used for downstream data downloading or table updating.
     """
-    query = _seq2boolquery(metadata_conditions,include_children)
+    query = _seq2boolquery(metadata_conditions,include_children,exlude_Unclassified)
     # get primary keys
     results = []
     next_token = None
@@ -627,12 +635,12 @@ def _para_get_column(arg_bag):
     primary_key, columns_to_get, col_filter = arg_bag
     consumed, return_row, next_token= client_global.get_row(_tablename, primary_key,columns_to_get,column_filter=col_filter)
     ### [0601] avoid errors from NA in the row
-    return [(x[0],x[1]) for x in return_row.primary_key] + [(x[0],x[1]) for x in return_row.attribute_columns]
-    #if not return_row is None:
+    if not return_row is None:
+        return [(x[0],x[1]) for x in return_row.primary_key] + [(x[0],x[1]) for x in return_row.attribute_columns]
     #   return ([x[1] for x in return_row.primary_key] + [x[1] for x in return_row.attribute_columns])
 
 
-def get_columnsbycell_para(rows_to_get,cols_to_get=None,col_filter=None,do_transfer = True, thread_num = multiprocessing.cpu_count()-1):
+def get_columnsbycell_para(rows_to_get=None,cols_to_get=None,col_filter=None,do_transfer = True, thread_num = multiprocessing.cpu_count()-1,exlude_Unclassified=False):
     """download the selected columns of the given cells from the OTS table on the server parallelly
     
         This function gets the cells in the given primary key list and downloads parts of columns or all columns. A further filtering on the gene expression levels can be conducted based on given column filters.
@@ -641,7 +649,7 @@ def get_columnsbycell_para(rows_to_get,cols_to_get=None,col_filter=None,do_trans
     Parameters
     ----------
     rows_to_get: list
-        rows_to_get is a list of primary keys of the cells to download. Each element in the list is a list containing a primary key tuple like [('cid',XXXXXXX)]
+        rows_to_get is a list of primary keys of the cells to download. Each element in the list is a list containing a primary key tuple like [('cid',XXXXXXX)]. The default value of this parameter is None which means to get all rows in the database.
 
     cols_to_get: list
         cols_to_get is a list of strings containing the names of the selected columns which can be either metadata columns or genes.
@@ -661,7 +669,9 @@ def get_columnsbycell_para(rows_to_get,cols_to_get=None,col_filter=None,do_trans
     thread_num: int
         Thread_num is number of threads used in the parallel upload process. The default value of this parameter is multiprocessing.cpu_count()-1 ,which means the function will use as many as accessible threads
 
-
+    exlude_Unclassified: bool
+        This parameter is only used when rows_to_get is None. It decides whether the cells with Unclassified cell_type will be included in the result.
+    
     Returns
     -------
     pandas::DataFrame/list
@@ -669,10 +679,13 @@ def get_columnsbycell_para(rows_to_get,cols_to_get=None,col_filter=None,do_trans
         
         If do_transfer == False, the return will be a list. Each element is a list which represents a cell.
     """
+    if rows_to_get == None:
+        rows_to_get = query_cells(exlude_Unclassified=exlude_Unclassified)
+    
     ## [0601] transform the hcad_name in the colnames into uHAF_name
     #if "hcad_name" in colnames:
     #    colnames[colnames.index("hcad_name")] = "uHAF_name"
-
+    
     arg_bag = [(rows_to_get[i],cols_to_get,col_filter) for i in range(len(rows_to_get))]
     try:
         with multiprocessing.Pool(thread_num) as pool:
@@ -697,7 +710,7 @@ def get_columnsbycell_para(rows_to_get,cols_to_get=None,col_filter=None,do_trans
     return df_get
 
 
-def get_columnsbycell(rows_to_get,cols_to_get=None,col_filter=None,do_transfer = True):
+def get_columnsbycell(rows_to_get=None,cols_to_get=None,col_filter=None,do_transfer = True,exlude_Unclassified=False):
     """download the selected columns of the given cells from the OTS table on the server (nonparallelly)
     
         This function gets the cells in the given primary key list and downloads parts of columns or all columns. A further filtering on the gene expression levels can be conducted based on given column filters.
@@ -708,7 +721,7 @@ def get_columnsbycell(rows_to_get,cols_to_get=None,col_filter=None,do_transfer =
     Parameters
     ----------
     rows_to_get: list
-        rows_to_get is a list of primary keys of the cells to download. Each element in the list is a list containing a primary key tuple like [('cid',XXXXXXX)]
+        rows_to_get is a list of primary keys of the cells to download. Each element in the list is a list containing a primary key tuple like [('cid',XXXXXXX)]. The default value of this parameter is None which means to get all rows in the database.
 
     cols_to_get: list
         cols_to_get is a list of strings containing the names of the selected columns which can be either metadata columns or genes
@@ -724,6 +737,9 @@ def get_columnsbycell(rows_to_get,cols_to_get=None,col_filter=None,do_transfer =
         If do_transfer is true, the output of this function will be transform into a pandas::DataFrame; vice versa.
         
         The default value of this parameter is True.
+    
+    exlude_Unclassified: bool
+        This parameter is only used when rows_to_get is None. It decides whether the cells with Unclassified cell_type will be included in the result.
 
 
     Returns
@@ -733,6 +749,11 @@ def get_columnsbycell(rows_to_get,cols_to_get=None,col_filter=None,do_transfer =
         
         If do_transfer == False, the return will be a list. Each element is a list which represents a cell.
     """
+    
+    if rows_to_get == None:
+        rows_to_get = query_cells(exlude_Unclassified)
+        
+        
     request = tablestore.BatchGetRowRequest()
     df_get = []
     colnames = None
@@ -762,7 +783,7 @@ def get_columnsbycell(rows_to_get,cols_to_get=None,col_filter=None,do_transfer =
         return df_get
 
 
-def get_column_set(rows_to_get, col_to_get, col_filter=None):
+def get_column_set(rows_to_get=None, col_to_get=None, col_filter=None,exlude_Unclassified=False):
     """get all unique values in a selected column in the given cells
     
     This function calls the get_columnsbycell_para function to download the values of a selected column in the given cells and return a set of all unique values.
@@ -770,7 +791,7 @@ def get_column_set(rows_to_get, col_to_get, col_filter=None):
     Parameters
     ----------
     rows_to_get: list
-        rows_to_get is a list of primary keys of the cells to download. Each element in the list is a list containing a primary key tuple like [('cid',XXXXXXX)]
+        rows_to_get is a list of primary keys of the cells to download. Each element in the list is a list containing a primary key tuple like [('cid',XXXXXXX)]. The default value of this parameter is None which means to get all rows in the database.
 
     col_to_get: list
         col_to_get is a list whose length is 1. The element in this list is the name of the selected column. if the length of the list is not 1, a error will be risen.
@@ -780,6 +801,8 @@ def get_column_set(rows_to_get, col_to_get, col_filter=None):
         
         Once a filter is set, the function will first filter the given cells then download the selected columns of the cells pass the filter. A massage will be print if no cells pass the given filter. The default value of this parameter is None that means no filtering will be conducted.
 
+    exlude_Unclassified: bool
+        This parameter is only used when rows_to_get is None. It decides whether the cells with Unclassified cell_type will be included in the result.
 
     Returns
     -------
@@ -787,6 +810,9 @@ def get_column_set(rows_to_get, col_to_get, col_filter=None):
         A set of all unique values in the selected column
     """
     if(len(col_to_get) == 1):
+        if rows_to_get == None:
+            rows_to_get = query_cells(exlude_Unclassified)
+        
         rows = get_columnsbycell_para(rows_to_get, col_to_get,do_transfer = False)
         s = set()
         for row in rows:
